import { registerCodeBlockRenderer } from ".";
import { SvelteCodeBlockRenderer } from "./SvelteCodeBlockRenderer";
import ReadingStatsCodeBlockView from "./ReadingStatsCodeBlockView.svelte";
import type BookTrackerPlugin from "@src/main";
import * as z from "zod/v4";
import { COLOR_NAMES } from "@utils/color";
import { BookMetadataSchema } from "@src/types";

export function registerReadingStatsCodeBlockProcessor(
	plugin: BookTrackerPlugin
): void {
	registerCodeBlockRenderer(
		plugin,
		"reading-stats",
		(source, el) =>
			new SvelteCodeBlockRenderer(
				ReadingStatsCodeBlockView,
				plugin,
				source,
				el
			)
	);
}

const PieGroupingSchema = z.object({
	label: z.string(),
	min: z.optional(z.number()),
	max: z.optional(z.number()),
});

export type PieGrouping = z.infer<typeof PieGroupingSchema>;

const color = z.union([z.literal("rainbow"), z.enum(COLOR_NAMES)]);

const PieChartColorSchema = z.union([
	color,
	z.array(z.enum(COLOR_NAMES)),
	z.array(z.object({ label: z.string(), color: z.enum(COLOR_NAMES) })),
]);

export type PieChartColor = z.infer<typeof PieChartColorSchema>;

const PieChart = z.object({
	type: z.literal("pie"),
	property: z.keyof(BookMetadataSchema),
	unit: z.optional(z.string()),
	unitPlural: z.optional(z.string()),
	groups: z.optional(z.array(PieGroupingSchema)),
	color: z.optional(PieChartColorSchema),
});

const BarChart = z.object({
	type: z.literal("bar"),
	property: z.keyof(BookMetadataSchema),
	horizontal: z.optional(z.boolean()),
	sortByLabel: z.optional(z.boolean()),
	topN: z.optional(z.number()),
	unit: z.optional(z.string()),
	unitPlural: z.optional(z.string()),
	color: z.optional(color),
});

const LineChart = z.object({
	type: z.literal("line"),
	property: z.keyof(BookMetadataSchema),
	unit: z.optional(z.string()),
	unitPlural: z.optional(z.string()),
	secondProperty: z.optional(z.keyof(BookMetadataSchema)),
	secondUnit: z.optional(z.string()),
	secondUnitPlural: z.optional(z.string()),
});

const BooksAndPagesChart = z.object({
	type: z.literal("books-and-pages"),
});

export const ReadingStatsSectionSchema = z.object({
	title: z.string(),
	stats: z.optional(
		z.array(
			z.discriminatedUnion("type", [
				z.object({
					type: z.enum(["count", "average", "total"]),
					label: z.string(),
					property: z.keyof(BookMetadataSchema),
				}),
				z.object({
					type: z.literal("book-count"),
					label: z.string(),
				}),
			])
		)
	),
	charts: z.array(
		z.discriminatedUnion("type", [
			PieChart,
			BarChart,
			LineChart,
			BooksAndPagesChart,
		])
	),
});

export type ReadingStatsSection = z.infer<typeof ReadingStatsSectionSchema>;
